@@ -46,6 +46,28 @@ def nested_a_result():
         return json.load(the_file)
 
 
+@pytest.fixture(scope='class')
+def nested_a_affected_paths():
+    return {
+        'root[0][0][2][0][1]', 'root[0][1][1][1][5]', 'root[0][2][1]',
+        'root[1][1][2][0][1]', 'root[1][2][0]', 'root[1][2][0][1][5]',
+        'root[1][0][2][2][3]', 'root[0][0][1][0][0]', 'root[0][1][0][2][3]',
+        'root[0][3][0][2][3]', 'root[0][3][1][0][2]', 'root[1][1][1][0][0]',
+        'root[1][0][1][2][1]', 'root[1][0][2][1][2]', 'root[1][3][0][2][3]',
+        'root[1][3][1][0][2]', 'root[1][2][0][2]', 'root[1][0][2][0][1]',
+        'root[0][3][2][0][1]', 'root[0][3][2][1][0]', 'root[1][3][1][1]',
+        'root[1][2][1][1][0]', 'root[1][2][1][0]', 'root[1][0][0][0][2]',
+        'root[1][3][2][1][0]', 'root[1][0][0][1][1]', 'root[0][1][2][0]',
+        'root[0][1][2][1][0]', 'root[0][2][0][1][2]', 'root[1][3][0][1]',
+        'root[0][3][1][1]', 'root[1][2][0][0][2]', 'root[1][3][2][0][1]',
+        'root[1][0][1][0]', 'root[1][2][0][0][0]', 'root[1][0][0][0][1]',
+        'root[1][3][2][2][2]', 'root[0][1][1][2][1]', 'root[0][1][1][2][2]',
+        'root[0][2][0][0][2]', 'root[0][2][0][0][3]', 'root[0][3][1][2][1]',
+        'root[0][3][1][2][2]', 'root[1][2][1][2][3]', 'root[1][0][0][1][2]',
+        'root[1][0][0][2][1]', 'root[1][3][1][2][1]', 'root[1][3][1][2][2]'
+    }
+
+
 @pytest.fixture(scope='class')
 def nested_b_t1():
     with open(os.path.join(FIXTURES_DIR, 'nested_b_t1.json')) as the_file:
