@@ -11,6 +11,11 @@ from deepdiff.diff import (
 from deepdiff import Delta, DeepSearch, extract as deep_extract
 from deepdiff.serialization import load_path_content, save_content_to_path
 
+try:
+    import orjson
+except ImportError:
+    orjson = None
+
 
 @click.group()
 def cli():
@@ -42,6 +47,7 @@ def cli():
 @click.option('--log-frequency-in-sec', required=False, default=0, type=int, show_default=True)
 @click.option('--max-passes', required=False, default=10000000, type=int, show_default=True)
 @click.option('--max_diffs', required=False, default=None, type=int, show_default=True)
+@click.option('--threshold-to-diff-deeper', required=False, default=0.33, type=float, show_default=False)
 @click.option('--number-format-notation', required=False, type=click.Choice(['f', 'e'], case_sensitive=True), show_default=True, default="f")
 @click.option('--progress-logger', required=False, type=click.Choice(['info', 'error'], case_sensitive=True), show_default=True, default="info")
 @click.option('--report-repetition', is_flag=True, show_default=True)
@@ -105,7 +111,13 @@ def diff(
         # printing into stdout
         sys.stdout.buffer.write(delta.dumps())
     else:
-        pprint(diff, indent=2)
+        try:
+            if orjson:
+                print(diff.to_json(option=orjson.OPT_INDENT_2))
+            else:
+                print(diff.to_json(indent=2))
+        except Exception:
+            pprint(diff, indent=2)
 
 
 @cli.command()
