@@ -1,17 +1,22 @@
 import sys
 import re
 import os
+import math
 import datetime
 import uuid
 import logging
 import warnings
 import string
 import time
+import enum
+from typing import NamedTuple, Any, List, Optional
 from ast import literal_eval
-from decimal import Decimal, localcontext
-from collections import namedtuple
+from decimal import Decimal, localcontext, InvalidOperation as InvalidDecimalOperation
 from itertools import repeat
-from ordered_set import OrderedSet
+# from orderly_set import OrderlySet as SetOrderedBase  # median: 0.806 s, some tests are failing
+# from orderly_set import SetOrdered as SetOrderedBase  # median 1.011 s, didn't work for tests
+from orderly_set import StableSetEq as SetOrderedBase  # median: 1.0867 s for cache test, 5.63s for all tests
+# from orderly_set import OrderedSet as SetOrderedBase  # median  1.1256 s for cache test, 5.63s for all tests
 from threading import Timer
 
 
@@ -19,6 +24,15 @@ class np_type:
     pass
 
 
+class pydantic_base_model_type:
+    pass
+
+
+class SetOrdered(SetOrderedBase):
+    def __repr__(self):
+        return str(list(self))
+
+
 try:
     import numpy as np
 except ImportError:  # pragma: no cover. The case without Numpy is tested locally only.
@@ -38,10 +52,11 @@ except ImportError:  # pragma: no cover. The case without Numpy is tested locall
     np_uintp = np_type  # pragma: no cover.
     np_float32 = np_type  # pragma: no cover.
     np_float64 = np_type  # pragma: no cover.
-    np_float_ = np_type  # pragma: no cover.
+    np_double = np_type  # pragma: no cover.
+    np_floating = np_type  # pragma: no cover.
     np_complex64 = np_type  # pragma: no cover.
     np_complex128 = np_type  # pragma: no cover.
-    np_complex_ = np_type  # pragma: no cover.
+    np_cdouble = np_type  # pragma: no cover.
     np_complexfloating = np_type  # pragma: no cover.
 else:
     np_array_factory = np.array
@@ -59,20 +74,21 @@ else:
     np_uintp = np.uintp
     np_float32 = np.float32
     np_float64 = np.float64
-    np_float_ = np.float_
+    np_double = np.double  # np.float_ is an alias for np.double and is being removed by NumPy 2.0
+    np_floating = np.floating
     np_complex64 = np.complex64
     np_complex128 = np.complex128
-    np_complex_ = np.complex_
+    np_cdouble = np.cdouble  # np.complex_ is an alias for np.cdouble and is being removed by NumPy 2.0
     np_complexfloating = np.complexfloating
 
 numpy_numbers = (
     np_int8, np_int16, np_int32, np_int64, np_uint8,
     np_uint16, np_uint32, np_uint64, np_intp, np_uintp,
-    np_float32, np_float64, np_float_, np_complex64,
-    np_complex128, np_complex_,)
+    np_float32, np_float64, np_double, np_floating, np_complex64,
+    np_complex128, np_cdouble,)
 
 numpy_complex_numbers = (
-    np_complexfloating, np_complex64, np_complex128, np_complex_,
+    np_complexfloating, np_complex64, np_complex128, np_cdouble,
 )
 
 numpy_dtypes = set(numpy_numbers)
@@ -82,6 +98,12 @@ numpy_dtype_str_to_type = {
     item.__name__: item for item in numpy_dtypes
 }
 
+try:
+    from pydantic.main import BaseModel as PydanticBaseModel
+except ImportError:
+    PydanticBaseModel = pydantic_base_model_type
+
+
 logger = logging.getLogger(__name__)
 
 py_major_version = sys.version_info.major
@@ -96,6 +118,51 @@ py4 = py_major_version == 4
 
 NUMERICS = frozenset(string.digits)
 
+
+class EnumBase(str, enum.Enum):
+    def __repr__(self):
+        """
+        We need to add a single quotes so we can easily copy the value when we do ipdb.
+        """
+        return f"'{self.name}'"
+
+    def __str__(self):
+        return self.name
+
+
+def _int_or_zero(value):
+    """
+    Tries to extract some number from a string.
+
+    12c becomes 12
+    """
+    try:
+        return int(value)
+    except Exception:
+        result = []
+        for char in value:
+            if char in NUMERICS:
+                result.append(char)
+        if result:
+            return int(''.join(result))
+        return 0
+
+
+def get_semvar_as_integer(version):
+    """
+    Converts:
+
+    '1.23.5' to 1023005
+    """
+    version = version.split('.')
+    if len(version) > 3:
+        version = version[:3]
+    elif len(version) < 3:
+        version.extend(['0'] * (3 - len(version)))
+
+    return sum([10**(i * 3) * _int_or_zero(v) for i, v in enumerate(reversed(version))])
+
+
 # we used to use OrderedDictPlus when dictionaries in Python were not ordered.
 dict_ = dict
 
@@ -108,18 +175,26 @@ if py2:  # pragma: no cover
 
 pypy3 = py3 and hasattr(sys, "pypy_translation_info")
 
+
+if np and get_semvar_as_integer(np.__version__) < 1019000:
+    sys.exit('The minimum required Numpy version is 1.19.0. Please upgrade your Numpy package.')
+
 strings = (str, bytes)  # which are both basestring
 unicode_type = str
 bytes_type = bytes
 only_complex_number = (complex,) + numpy_complex_numbers
 only_numbers = (int, float, complex, Decimal) + numpy_numbers
 datetimes = (datetime.datetime, datetime.date, datetime.timedelta, datetime.time)
-uuids = (uuid.UUID)
+uuids = (uuid.UUID, )
 times = (datetime.datetime, datetime.time)
 numbers = only_numbers + datetimes
 booleans = (bool, np_bool_)
 
-IndexedHash = namedtuple('IndexedHash', 'indexes item')
+basic_types = strings + numbers + uuids + booleans + (type(None), )
+
+class IndexedHash(NamedTuple):
+    indexes: List
+    item: Any
 
 current_dir = os.path.dirname(os.path.abspath(__file__))
 
@@ -131,7 +206,7 @@ TREE_VIEW = 'tree'
 TEXT_VIEW = 'text'
 DELTA_VIEW = '_delta'
 
-ENUM_IGNORE_KEYS = frozenset(['_name_', '_value_', '_sort_order_'])
+ENUM_INCLUDE_KEYS = ['__objclass__', 'name', 'value']
 
 
 def short_repr(item, max_length=15):
@@ -252,7 +327,7 @@ def add_root_to_paths(paths):
     """
     if paths is None:
         return
-    result = OrderedSet()
+    result = SetOrdered()
     for path in paths:
         if path.startswith('root'):
             result.add(path)
@@ -307,8 +382,8 @@ def type_in_type_group(item, type_group):
 
 def type_is_subclass_of_type_group(item, type_group):
     return isinstance(item, type_group) \
-           or (isinstance(item, type) and issubclass(item, type_group)) \
-           or type_in_type_group(item, type_group)
+        or (isinstance(item, type) and issubclass(item, type_group)) \
+        or type_in_type_group(item, type_group)
 
 
 def get_doc(doc_filename):
@@ -334,7 +409,7 @@ def number_to_string(number, significant_digits, number_format_notation="f"):
         using = number_formatting[number_format_notation]
     except KeyError:
         raise ValueError("number_format_notation got invalid value of {}. The valid values are 'f' and 'e'".format(number_format_notation)) from None
-    
+
     if not isinstance(number, numbers):
         return number
     elif isinstance(number, Decimal):
@@ -342,7 +417,13 @@ def number_to_string(number, significant_digits, number_format_notation="f"):
             # Precision = number of integer digits + significant_digits
             # Using number//1 to get the integer part of the number
             ctx.prec = len(str(abs(number // 1))) + significant_digits
-            number = number.quantize(Decimal('0.' + '0' * significant_digits))
+            try:
+                number = number.quantize(Decimal('0.' + '0' * significant_digits))
+            except InvalidDecimalOperation:
+                # Sometimes rounding up causes a higher precision to be needed for the quantize operation
+                # For example '999.99999999' will become '1000.000000' after quantize
+                ctx.prec += 1
+                number = number.quantize(Decimal('0.' + '0' * significant_digits))
     elif isinstance(number, only_complex_number):
         # Case for complex numbers.
         number = number.__class__(
@@ -360,6 +441,7 @@ def number_to_string(number, significant_digits, number_format_notation="f"):
             )
         )
     else:
+        # import pytest; pytest.set_trace()
         number = round(number=number, ndigits=significant_digits)
 
         if significant_digits == 0:
@@ -451,31 +533,6 @@ not_found = _NotFound()
 warnings.simplefilter('once', DeepDiffDeprecationWarning)
 
 
-class OrderedSetPlus(OrderedSet):
-
-    def lpop(self):
-        """
-        Remove and return the first element from the set.
-        Raises KeyError if the set is empty.
-        Example:
-            >>> oset = OrderedSet([1, 2, 3])
-            >>> oset.lpop()
-            1
-        """
-        if not self.items:
-            raise KeyError('lpop from an empty set')
-
-        elem = self.items[0]
-        del self.items[0]
-        del self.map[elem]
-        return elem
-
-    def __repr__(self):
-        return str(list(self))
-
-    __str__ = __repr__
-
-
 class RepeatedTimer:
     """
     Threaded Repeated Timer by MestreLion
@@ -603,7 +660,7 @@ def diff_numpy_array(A, B):
     By Divakar
     https://stackoverflow.com/a/52417967/1497443
     """
-    return A[~np.in1d(A, B)]
+    return A[~np.isin(A, B)]
 
 
 PYTHON_TYPE_TO_NUMPY_TYPE = {
@@ -628,27 +685,119 @@ def get_homogeneous_numpy_compatible_type_of_seq(seq):
         return False
 
 
-def detailed__dict__(obj, ignore_private_variables=True, ignore_keys=frozenset()):
+def detailed__dict__(obj, ignore_private_variables=True, ignore_keys=frozenset(), include_keys=None):
     """
     Get the detailed dictionary of an object.
 
     This is used so we retrieve object properties too.
     """
-    result = obj.__dict__.copy()  # A shallow copy
-    private_var_prefix = f"_{obj.__class__.__name__}__"  # The semi private variables in Python get this prefix
-    for key in ignore_keys:
-        if key in result or (
-            ignore_private_variables and key.startswith('__') and not key.startswith(private_var_prefix)
-        ):
-            del result[key]
-    for key in dir(obj):
-        if key not in result and key not in ignore_keys and (
-                not ignore_private_variables or (
-                    ignore_private_variables and not key.startswith('__') and not key.startswith(private_var_prefix)
-                )
-        ):
-            value = getattr(obj, key)
-            if not callable(value):
-                print(f"{key}: {value}")
-                result[key] = value
+    if include_keys:
+        result = {}
+        for key in include_keys:
+            try:
+                value = getattr(obj, key)
+            except Exception:
+                pass
+            else:
+                if not callable(value) or key == '__objclass__':  # We don't want to compare functions, however for backward compatibility, __objclass__ needs to be reported.
+                    result[key] = value
+    else:
+        result = obj.__dict__.copy()  # A shallow copy
+        private_var_prefix = f"_{obj.__class__.__name__}__"  # The semi private variables in Python get this prefix
+        for key in ignore_keys:
+            if key in result or (
+                ignore_private_variables and key.startswith('__') and not key.startswith(private_var_prefix)
+            ):
+                del result[key]
+        for key in dir(obj):
+            if key not in result and key not in ignore_keys and (
+                    not ignore_private_variables or (
+                        ignore_private_variables and not key.startswith('__') and not key.startswith(private_var_prefix)
+                    )
+            ):
+                value = getattr(obj, key)
+                if not callable(value):
+                    result[key] = value
     return result
+
+
+def named_tuple_repr(self):
+    fields = []
+    for field, value in self._asdict().items():
+        # Only include fields that do not have their default value
+        if field in self._field_defaults:
+            if value != self._field_defaults[field]:
+                fields.append(f"{field}={value!r}")
+        else:
+            fields.append(f"{field}={value!r}")
+
+    return f"{self.__class__.__name__}({', '.join(fields)})"
+
+
+class OpcodeTag(EnumBase):
+    insert = 'insert'
+    delete = 'delete'
+    equal = 'equal'
+    replace = 'replace'
+    # swapped = 'swapped'  # in the future we should support reporting of items swapped with each other
+
+
+class Opcode(NamedTuple):
+    tag: str
+    t1_from_index: int
+    t1_to_index: int
+    t2_from_index: int
+    t2_to_index: int
+    old_values: Optional[List[Any]] = None
+    new_values: Optional[List[Any]] = None
+
+    __repr__ = __str__ = named_tuple_repr
+
+
+class FlatDataAction(EnumBase):
+    values_changed = 'values_changed'
+    type_changes = 'type_changes'
+    set_item_added = 'set_item_added'
+    set_item_removed = 'set_item_removed'
+    dictionary_item_added = 'dictionary_item_added'
+    dictionary_item_removed = 'dictionary_item_removed'
+    iterable_item_added = 'iterable_item_added'
+    iterable_item_removed = 'iterable_item_removed'
+    iterable_item_moved = 'iterable_item_moved'
+    iterable_items_inserted = 'iterable_items_inserted'  # opcode
+    iterable_items_deleted = 'iterable_items_deleted'  # opcode
+    iterable_items_replaced = 'iterable_items_replaced'  # opcode
+    iterable_items_equal = 'iterable_items_equal'  # opcode
+    attribute_removed = 'attribute_removed'
+    attribute_added = 'attribute_added'
+    unordered_iterable_item_added = 'unordered_iterable_item_added'
+    unordered_iterable_item_removed = 'unordered_iterable_item_removed'
+
+
+OPCODE_TAG_TO_FLAT_DATA_ACTION = {
+    OpcodeTag.insert: FlatDataAction.iterable_items_inserted,
+    OpcodeTag.delete: FlatDataAction.iterable_items_deleted,
+    OpcodeTag.replace: FlatDataAction.iterable_items_replaced,
+    OpcodeTag.equal: FlatDataAction.iterable_items_equal,
+}
+
+FLAT_DATA_ACTION_TO_OPCODE_TAG = {v: i for i, v in OPCODE_TAG_TO_FLAT_DATA_ACTION.items()}
+
+
+UnkownValueCode = 'unknown___'
+
+
+class FlatDeltaRow(NamedTuple):
+    path: List
+    action: FlatDataAction
+    value: Optional[Any] = UnkownValueCode
+    old_value: Optional[Any] = UnkownValueCode
+    type: Optional[Any] = UnkownValueCode
+    old_type: Optional[Any] = UnkownValueCode
+    new_path: Optional[List] = None
+    t1_from_index: Optional[int] = None
+    t1_to_index: Optional[int] = None
+    t2_from_index: Optional[int] = None
+    t2_to_index: Optional[int] = None
+
+    __repr__ = __str__ = named_tuple_repr
