@@ -1,10 +1,10 @@
 import logging
 from collections.abc import Mapping
 from copy import copy
-from ordered_set import OrderedSet
 from deepdiff.helper import (
     RemapDict, strings, short_repr, notpresent, get_type, numpy_numbers, np, literal_eval_extended,
-    dict_)
+    dict_, SetOrdered)
+from deepdiff.path import stringify_element
 
 logger = logging.getLogger(__name__)
 
@@ -41,26 +41,16 @@ class ResultDict(RemapDict):
         Remove empty keys from this object. Should always be called after the result is final.
         :return:
         """
-        empty_keys = [k for k, v in self.items() if not v]
+        empty_keys = [k for k, v in self.items() if not isinstance(v, (int)) and not v]
 
         for k in empty_keys:
             del self[k]
 
 
-class PrettyOrderedSet(OrderedSet):
-    """
-    From the perspective of the users of the library, they are dealing with lists.
-    Behind the scene, we have ordered sets.
-    """
-
-    def __repr__(self):
-        return '[{}]'.format(", ".join(map(str, self)))
-
-
 class TreeResult(ResultDict):
     def __init__(self):
         for key in REPORT_KEYS:
-            self[key] = PrettyOrderedSet()
+            self[key] = SetOrdered()
 
     def mutual_add_removes_to_become_value_changes(self):
         """
@@ -78,7 +68,7 @@ class TreeResult(ResultDict):
             mutual_paths = set(added_paths) & set(removed_paths)
 
             if mutual_paths and 'values_changed' not in self:
-                self['values_changed'] = PrettyOrderedSet()
+                self['values_changed'] = SetOrdered()
             for path in mutual_paths:
                 level_before = removed_paths[path]
                 self['iterable_item_removed'].remove(level_before)
@@ -86,6 +76,7 @@ class TreeResult(ResultDict):
                 self['iterable_item_added'].remove(level_after)
                 level_before.t2 = level_after.t2
                 self['values_changed'].add(level_before)
+                level_before.report_type = 'values_changed'
         if 'iterable_item_removed' in self and not self['iterable_item_removed']:
             del self['iterable_item_removed']
         if 'iterable_item_added' in self and not self['iterable_item_added']:
@@ -93,9 +84,18 @@ class TreeResult(ResultDict):
 
     def __getitem__(self, item):
         if item not in self:
-            self[item] = PrettyOrderedSet()
+            self[item] = SetOrdered()
         return self.get(item)
 
+    def __len__(self):
+        length = 0
+        for value in self.values():
+            if isinstance(value, SetOrdered):
+                length += len(value)
+            elif isinstance(value, int):
+                length += 1
+        return length
+
 
 class TextResult(ResultDict):
     ADD_QUOTES_TO_STRINGS = True
@@ -114,8 +114,8 @@ class TextResult(ResultDict):
             "iterable_item_moved": dict_(),
             "attribute_added": self.__set_or_dict(),
             "attribute_removed": self.__set_or_dict(),
-            "set_item_removed": PrettyOrderedSet(),
-            "set_item_added": PrettyOrderedSet(),
+            "set_item_removed": SetOrdered(),
+            "set_item_added": SetOrdered(),
             "repetition_change": dict_()
         })
 
@@ -123,7 +123,7 @@ class TextResult(ResultDict):
             self._from_tree_results(tree_results)
 
     def __set_or_dict(self):
-        return {} if self.verbose_level >= 2 else PrettyOrderedSet()
+        return {} if self.verbose_level >= 2 else SetOrdered()
 
     def _from_tree_results(self, tree):
         """
@@ -147,9 +147,17 @@ class TextResult(ResultDict):
         self._from_tree_deep_distance(tree)
         self._from_tree_custom_results(tree)
 
-    def _from_tree_default(self, tree, report_type):
+    def _from_tree_default(self, tree, report_type, ignore_if_in_iterable_opcodes=False):
         if report_type in tree:
+                
             for change in tree[report_type]:  # report each change
+                # When we convert from diff to delta result, we care more about opcodes than iterable_item_added or removed
+                if (
+                    ignore_if_in_iterable_opcodes
+                    and report_type in {"iterable_item_added", "iterable_item_removed"}
+                    and change.up.path(force=FORCE_DEFAULT) in self["_iterable_opcodes"]
+                ):
+                    continue
                 # determine change direction (added or removed)
                 # Report t2 (the new one) whenever possible.
                 # In cases where t2 doesn't exist (i.e. stuff removed), report t1.
@@ -160,7 +168,7 @@ class TextResult(ResultDict):
 
                 # do the reporting
                 report = self[report_type]
-                if isinstance(report, PrettyOrderedSet):
+                if isinstance(report, SetOrdered):
                     report.add(change.path(force=FORCE_DEFAULT))
                 elif isinstance(report, dict):
                     report[change.path(force=FORCE_DEFAULT)] = item
@@ -175,6 +183,7 @@ class TextResult(ResultDict):
     def _from_tree_type_changes(self, tree):
         if 'type_changes' in tree:
             for change in tree['type_changes']:
+                path = change.path(force=FORCE_DEFAULT)
                 if type(change.t1) is type:
                     include_values = False
                     old_type = change.t1
@@ -185,19 +194,26 @@ class TextResult(ResultDict):
                     new_type = get_type(change.t2)
                 remap_dict = RemapDict({
                     'old_type': old_type,
-                    'new_type': new_type
+                    'new_type': new_type,
                 })
-                self['type_changes'][change.path(
-                    force=FORCE_DEFAULT)] = remap_dict
+                if self.verbose_level > 1:
+                    new_path = change.path(use_t2=True, force=FORCE_DEFAULT)
+                    if path != new_path:
+                        remap_dict['new_path'] = new_path
+                self['type_changes'][path] = remap_dict
                 if self.verbose_level and include_values:
                     remap_dict.update(old_value=change.t1, new_value=change.t2)
 
     def _from_tree_value_changed(self, tree):
         if 'values_changed' in tree and self.verbose_level > 0:
             for change in tree['values_changed']:
+                path = change.path(force=FORCE_DEFAULT)
                 the_changed = {'new_value': change.t2, 'old_value': change.t1}
-                self['values_changed'][change.path(
-                    force=FORCE_DEFAULT)] = the_changed
+                if self.verbose_level > 1:
+                    new_path = change.path(use_t2=True, force=FORCE_DEFAULT)
+                    if path != new_path:
+                        the_changed['new_path'] = new_path
+                self['values_changed'][path] = the_changed
                 if 'diff' in change.additional:
                     the_changed.update({'diff': change.additional['diff']})
 
@@ -254,7 +270,7 @@ class TextResult(ResultDict):
     def _from_tree_custom_results(self, tree):
         for k, _level_list in tree.items():
             if k not in REPORT_KEYS:
-                if not isinstance(_level_list, PrettyOrderedSet):
+                if not isinstance(_level_list, SetOrdered):
                     continue
 
                 # if len(_level_list) == 0:
@@ -274,8 +290,9 @@ class TextResult(ResultDict):
 class DeltaResult(TextResult):
     ADD_QUOTES_TO_STRINGS = False
 
-    def __init__(self, tree_results=None, ignore_order=None):
+    def __init__(self, tree_results=None, ignore_order=None, always_include_values=False, _iterable_opcodes=None):
         self.ignore_order = ignore_order
+        self.always_include_values = always_include_values
 
         self.update({
             "type_changes": dict_(),
@@ -291,6 +308,7 @@ class DeltaResult(TextResult):
             "set_item_added": dict_(),
             "iterable_items_added_at_indexes": dict_(),
             "iterable_items_removed_at_indexes": dict_(),
+            "_iterable_opcodes": _iterable_opcodes or {},
         })
 
         if tree_results:
@@ -312,8 +330,8 @@ class DeltaResult(TextResult):
             self._from_tree_iterable_item_added_or_removed(
                 tree, 'iterable_item_removed', delta_report_key='iterable_items_removed_at_indexes')
         else:
-            self._from_tree_default(tree, 'iterable_item_added')
-            self._from_tree_default(tree, 'iterable_item_removed')
+            self._from_tree_default(tree, 'iterable_item_added', ignore_if_in_iterable_opcodes=True)
+            self._from_tree_default(tree, 'iterable_item_removed', ignore_if_in_iterable_opcodes=True)
             self._from_tree_iterable_item_moved(tree)
         self._from_tree_default(tree, 'attribute_added')
         self._from_tree_default(tree, 'attribute_removed')
@@ -364,21 +382,27 @@ class DeltaResult(TextResult):
                     except Exception:
                         pass
 
+                path = change.path(force=FORCE_DEFAULT)
+                new_path = change.path(use_t2=True, force=FORCE_DEFAULT)
                 remap_dict = RemapDict({
                     'old_type': old_type,
-                    'new_type': new_type
+                    'new_type': new_type,
                 })
-                self['type_changes'][change.path(
-                    force=FORCE_DEFAULT)] = remap_dict
-                if include_values:
+                if path != new_path:
+                    remap_dict['new_path'] = new_path
+                self['type_changes'][path] = remap_dict
+                if include_values or self.always_include_values:
                     remap_dict.update(old_value=change.t1, new_value=change.t2)
 
     def _from_tree_value_changed(self, tree):
         if 'values_changed' in tree:
             for change in tree['values_changed']:
+                path = change.path(force=FORCE_DEFAULT)
+                new_path = change.path(use_t2=True, force=FORCE_DEFAULT)
                 the_changed = {'new_value': change.t2, 'old_value': change.t1}
-                self['values_changed'][change.path(
-                    force=FORCE_DEFAULT)] = the_changed
+                if path != new_path:
+                    the_changed['new_path'] = new_path
+                self['values_changed'][path] = the_changed
                 # If we ever want to store the difflib results instead of the new_value
                 # these lines need to be uncommented and the Delta object needs to be able
                 # to use them.
@@ -401,9 +425,12 @@ class DeltaResult(TextResult):
     def _from_tree_iterable_item_moved(self, tree):
         if 'iterable_item_moved' in tree:
             for change in tree['iterable_item_moved']:
-                the_changed = {'new_path': change.path(use_t2=True), 'value': change.t2}
-                self['iterable_item_moved'][change.path(
-                    force=FORCE_DEFAULT)] = the_changed
+                if (
+                    change.up.path(force=FORCE_DEFAULT) not in self["_iterable_opcodes"]
+                ):
+                    the_changed = {'new_path': change.path(use_t2=True), 'value': change.t2}
+                    self['iterable_item_moved'][change.path(
+                        force=FORCE_DEFAULT)] = the_changed
 
 
 class DiffLevel:
@@ -573,6 +600,10 @@ class DiffLevel:
         else:
             self.__dict__[key] = value
 
+    def __iter__(self):
+        yield self.t1
+        yield self.t2
+
     @property
     def repetition(self):
         return self.additional['repetition']
@@ -634,7 +665,9 @@ class DiffLevel:
         else:
             next_rel = root_level.t1_child_rel or root_level.t2_child_rel  # next relationship object to get a formatted param from
 
-        return next_rel.param
+        if next_rel:
+            return next_rel.param
+        return notpresent
 
     def path(self, root="root", force=None, get_parent_too=False, use_t2=False, output_format='str'):
         """
@@ -683,8 +716,8 @@ class DiffLevel:
         # traverse all levels of this relationship
         while level and level is not self:
             # get this level's relationship object
-            if(use_t2):
-                next_rel = level.t2_child_rel
+            if use_t2:
+                next_rel = level.t2_child_rel or level.t1_child_rel
             else:
                 next_rel = level.t1_child_rel or level.t2_child_rel  # next relationship object to get a formatted param from
 
@@ -866,9 +899,12 @@ class ChildRelationship:
         """
         param = self.param
         if isinstance(param, strings):
-            result = param if self.quote_str is None else self.quote_str.format(param)
+            result = stringify_element(param, quote_str=self.quote_str)
         elif isinstance(param, tuple):  # Currently only for numpy ndarrays
             result = ']['.join(map(repr, param))
+        elif hasattr(param, '__dataclass_fields__'):
+            attrs_to_values = [f"{key}={value}" for key, value in [(i, getattr(param, i)) for i in param.__dataclass_fields__]]
+            result = f"{param.__class__.__name__}({','.join(attrs_to_values)})"
         else:
             candidate = repr(param)
             try:
