@@ -9,7 +9,7 @@ import re  # NOQA
 import builtins  # NOQA
 import datetime  # NOQA
 import decimal  # NOQA
-import ordered_set  # NOQA
+import orderly_set  # NOQA
 import collections  # NOQA
 try:
     import yaml
@@ -41,11 +41,21 @@ try:
 except ImportError:  # pragma: no cover.
     PydanticBaseModel = None
 
-from copy import deepcopy
+from copy import deepcopy, copy
 from functools import partial
 from collections.abc import Mapping
+from typing import Callable, Optional, Union
 from deepdiff.helper import (
-    strings, get_type, TEXT_VIEW, np_float32, np_float64, np_int32, np_int64
+    strings,
+    get_type,
+    TEXT_VIEW,
+    np_float32,
+    np_float64,
+    np_int32,
+    np_int64,
+    np_ndarray,
+    Opcode,
+    SetOrdered,
 )
 from deepdiff.model import DeltaResult
 
@@ -92,10 +102,14 @@ SAFE_TO_IMPORT = {
     'datetime.timedelta',
     'decimal.Decimal',
     'uuid.UUID',
-    'ordered_set.OrderedSet',
+    'orderly_set.sets.OrderedSet',
+    'orderly_set.sets.OrderlySet',
+    'orderly_set.sets.StableSetEq',
+    'deepdiff.helper.SetOrdered',
     'collections.namedtuple',
     'collections.OrderedDict',
     're.Pattern',
+    'deepdiff.helper.Opcode',
 }
 
 
@@ -120,7 +134,7 @@ TYPE_STR_TO_TYPE = {
     'time': datetime.time,
     'timedelta': datetime.timedelta,
     'Decimal': decimal.Decimal,
-    'OrderedSet': ordered_set.OrderedSet,
+    'SetOrdered': SetOrdered,
     'namedtuple': collections.namedtuple,
     'OrderedDict': collections.OrderedDict,
     'Pattern': re.Pattern,    
@@ -166,7 +180,7 @@ class SerializationMixin:
         else:
             logger.error('jsonpickle library needs to be installed in order to run from_json_pickle')  # pragma: no cover. Json pickle is getting deprecated.
 
-    def to_json(self, default_mapping=None, **kwargs):
+    def to_json(self, default_mapping: Optional[dict]=None, force_use_builtin_json=False, **kwargs):
         """
         Dump json of the text view.
         **Parameters**
@@ -177,6 +191,11 @@ class SerializationMixin:
         If you have a certain object type that the json serializer can not serialize it, please pass the appropriate type
         conversion through this dictionary.
 
+        force_use_builtin_json: Boolean, default = False
+            When True, we use Python's builtin Json library for serialization,
+            even if Orjson is installed.
+
+
         kwargs: Any other kwargs you pass will be passed on to Python's json.dumps()
 
         **Example**
@@ -199,7 +218,12 @@ class SerializationMixin:
             '{"type_changes": {"root": {"old_type": "A", "new_type": "B", "old_value": "obj A", "new_value": "obj B"}}}'
         """
         dic = self.to_dict(view_override=TEXT_VIEW)
-        return json_dumps(dic, default_mapping=default_mapping, **kwargs)
+        return json_dumps(
+            dic,
+            default_mapping=default_mapping,
+            force_use_builtin_json=force_use_builtin_json,
+            **kwargs,
+        )
 
     def to_dict(self, view_override=None):
         """
@@ -241,7 +265,29 @@ class SerializationMixin:
         if self.group_by is not None:
             raise ValueError(DELTA_ERROR_WHEN_GROUP_BY)
 
-        result = DeltaResult(tree_results=self.tree, ignore_order=self.ignore_order, always_include_values=always_include_values)
+        if directed and not always_include_values:
+            _iterable_opcodes = {}
+            for path, op_codes in self._iterable_opcodes.items():
+                _iterable_opcodes[path] = []
+                for op_code in op_codes:
+                    new_op_code = Opcode(
+                        tag=op_code.tag,
+                        t1_from_index=op_code.t1_from_index,
+                        t1_to_index=op_code.t1_to_index,
+                        t2_from_index=op_code.t2_from_index,
+                        t2_to_index=op_code.t2_to_index,
+                        new_values=op_code.new_values,
+                    )
+                    _iterable_opcodes[path].append(new_op_code)
+        else:
+            _iterable_opcodes = self._iterable_opcodes
+
+        result = DeltaResult(
+            tree_results=self.tree,
+            ignore_order=self.ignore_order,
+            always_include_values=always_include_values,
+            _iterable_opcodes=_iterable_opcodes,
+        )
         result.remove_empty_keys()
         if report_repetition_required and self.ignore_order and not self.report_repetition:
             raise ValueError(DELTA_IGNORE_ORDER_NEEDS_REPETITION_REPORT)
@@ -261,11 +307,13 @@ class SerializationMixin:
 
         return deepcopy(dict(result))
 
-    def pretty(self):
+    def pretty(self, prefix: Optional[Union[str, Callable]]=None):
         """
         The pretty human readable string output for the diff object
         regardless of what view was used to generate the diff.
 
+        prefix can be a callable or a string or None.
+
         Example:
             >>> t1={1,2,4}
             >>> t2={2,3}
@@ -275,12 +323,16 @@ class SerializationMixin:
             Item root[1] removed from set.
         """
         result = []
+        if prefix is None:
+            prefix = ''
         keys = sorted(self.tree.keys())  # sorting keys to guarantee constant order across python versions.
         for key in keys:
             for item_key in self.tree[key]:
                 result += [pretty_print_diff(item_key)]
 
-        return '\n'.join(result)
+        if callable(prefix):
+            return "\n".join(f"{prefix(diff=self)}{r}" for r in result)
+        return "\n".join(f"{prefix}{r}" for r in result)
 
 
 class _RestrictedUnpickler(pickle.Unpickler):
@@ -537,9 +589,16 @@ def _serialize_decimal(value):
         return float(value)
 
 
+def _serialize_tuple(value):
+    if hasattr(value, '_asdict'):  # namedtuple
+        return value._asdict()
+    return value
+
+
 JSON_CONVERTOR = {
     decimal.Decimal: _serialize_decimal,
-    ordered_set.OrderedSet: list,
+    SetOrdered: list,
+    orderly_set.StableSetEq: list,
     set: list,
     type: lambda x: x.__name__,
     bytes: lambda x: x.decode('utf-8'),
@@ -548,7 +607,10 @@ JSON_CONVERTOR = {
     np_float32: float,
     np_float64: float,
     np_int32: int,
-    np_int64: int
+    np_int64: int,
+    np_ndarray: lambda x: x.tolist(),
+    tuple: _serialize_tuple,
+    Mapping: dict,
 }
 
 if PydanticBaseModel:
@@ -566,6 +628,9 @@ def json_convertor_default(default_mapping=None):
         for original_type, convert_to in _convertor_mapping.items():
             if isinstance(obj, original_type):
                 return convert_to(obj)
+        # This is to handle reverse() which creates a generator of type list_reverseiterator
+        if obj.__class__.__name__ == 'list_reverseiterator':
+            return list(copy(obj))
         raise TypeError('We do not know how to convert {} of type {} for json serialization. Please pass the default_mapping parameter with proper mapping of the object to a basic python type.'.format(obj, type(obj)))
 
     return _convertor
@@ -585,14 +650,26 @@ class JSONDecoder(json.JSONDecoder):
         return obj
 
 
-def json_dumps(item, default_mapping=None, **kwargs):
+def json_dumps(item, default_mapping=None, force_use_builtin_json: bool=False, **kwargs):
     """
     Dump json with extra details that are not normally json serializable
+
+    parameters
+    ----------
+
+    force_use_builtin_json: Boolean, default = False
+        When True, we use Python's builtin Json library for serialization,
+        even if Orjson is installed.
     """
-    if orjson:
+    if orjson and not force_use_builtin_json:
         indent = kwargs.pop('indent', None)
         if indent:
             kwargs['option'] = orjson.OPT_INDENT_2
+        if 'sort_keys' in kwargs:
+            raise TypeError(
+                "orjson does not accept the sort_keys parameter. "
+                "If you need to pass sort_keys, set force_use_builtin_json=True "
+                "to use Python's built-in json library instead of orjson.")
         return orjson.dumps(
             item,
             default=json_convertor_default(default_mapping=default_mapping),
