@@ -10,13 +10,12 @@ from pickle import UnpicklingError
 from decimal import Decimal
 from collections import Counter
 from deepdiff import DeepDiff
-from deepdiff.helper import pypy3, py_current_version, np_ndarray, Opcode
+from deepdiff.helper import pypy3, py_current_version, np_ndarray, Opcode, SetOrdered
 from deepdiff.serialization import (
     pickle_load, pickle_dump, ForbiddenModule, ModuleNotFoundError,
     MODULE_NOT_FOUND_MSG, FORBIDDEN_MODULE_MSG, pretty_print_diff,
     load_path_content, UnsupportedFormatErr, json_dumps, json_loads)
 from conftest import FIXTURES_DIR
-from ordered_set import OrderedSet
 from tests import PicklableClass
 
 import logging
@@ -48,6 +47,14 @@ class TestSerialization:
         jsoned = ddiff.to_json()
         assert "world" in jsoned
 
+    def test_serialization_text_force_builtin_json(self):
+        ddiff = DeepDiff(t1, t2)
+        with pytest.raises(TypeError) as excinfo:
+            jsoned = ddiff.to_json(sort_keys=True)
+        assert str(excinfo.value).startswith("orjson does not accept the sort_keys parameter.")
+        jsoned = ddiff.to_json(sort_keys=True, force_use_builtin_json=True)
+        assert "world" in jsoned
+
     def test_deserialization(self):
         ddiff = DeepDiff(t1, t2)
         jsoned = ddiff.to_json_pickle()
@@ -133,7 +140,7 @@ class TestLoadContet:
 class TestPickling:
 
     def test_serialize(self):
-        obj = [1, 2, 3, None, {10: 11E2}, frozenset(['a', 'c']), OrderedSet([2, 1]),
+        obj = [1, 2, 3, None, {10: 11E2}, frozenset(['a', 'c']), SetOrdered([2, 1]),
                datetime.datetime(2022, 4, 10, 0, 40, 41, 357857), datetime.time(11), Decimal('11.2'), 123.11]
         serialized = pickle_dump(obj)
         loaded = pickle_load(serialized)
@@ -199,7 +206,7 @@ class TestDeepDiffPretty:
     def test_pretty_print_diff_type_changes(self, t1, t2, item_path, old_type, new_type, old_val_displayed,
                                             new_val_displayed):
         ddiff = DeepDiff(t1, t2, view='tree')
-        result = pretty_print_diff(ddiff.tree['type_changes'].items[0])
+        result = pretty_print_diff(ddiff.tree['type_changes'][0])
         assert result == 'Type of {} changed from {} to {} and value changed from {} to {}.'.format(item_path, old_type, new_type, old_val_displayed, new_val_displayed)
 
     @pytest.mark.parametrize('t1, t2, item_path, verbose_level',
@@ -213,7 +220,7 @@ class TestDeepDiffPretty:
                              ])
     def test_pretty_print_diff_dictionary_item_added(self, t1, t2, item_path, verbose_level):
         ddiff = DeepDiff(t1, t2, view='tree', verbose_level=verbose_level)
-        result = pretty_print_diff(ddiff.tree['dictionary_item_added'].items[0])
+        result = pretty_print_diff(ddiff.tree['dictionary_item_added'][0])
         assert result == 'Item {} added to dictionary.'.format(item_path)
 
     @pytest.mark.parametrize('t1, t2, item_path, verbose_level',
@@ -227,7 +234,7 @@ class TestDeepDiffPretty:
                              ])
     def test_pretty_print_diff_dictionary_item_removed(self, t1, t2, item_path, verbose_level):
         ddiff = DeepDiff(t1, t2, view='tree', verbose_level=verbose_level)
-        result = pretty_print_diff(ddiff.tree['dictionary_item_removed'].items[0])
+        result = pretty_print_diff(ddiff.tree['dictionary_item_removed'][0])
         assert result == 'Item {} removed from dictionary.'.format(item_path)
 
     @pytest.mark.parametrize('t1, t2, item_path, old_val_displayed, new_val_displayed',
@@ -237,7 +244,7 @@ class TestDeepDiffPretty:
                              ])
     def test_pretty_print_diff_values_changed(self, t1, t2, item_path, old_val_displayed, new_val_displayed):
         ddiff = DeepDiff(t1, t2, view='tree')
-        result = pretty_print_diff(ddiff.tree['values_changed'].items[0])
+        result = pretty_print_diff(ddiff.tree['values_changed'][0])
         assert result == 'Value of {} changed from {} to {}.'.format(item_path, old_val_displayed, new_val_displayed)
 
     @pytest.mark.parametrize('t1, t2, item_path, verbose_level',
@@ -249,7 +256,7 @@ class TestDeepDiffPretty:
                              ])
     def test_pretty_print_diff_iterable_item_added(self, t1, t2, item_path, verbose_level):
         ddiff = DeepDiff(t1, t2, view='tree', verbose_level=verbose_level)
-        result = pretty_print_diff(ddiff.tree['iterable_item_added'].items[0])
+        result = pretty_print_diff(ddiff.tree['iterable_item_added'][0])
         assert result == 'Item {} added to iterable.'.format(item_path)
 
     @pytest.mark.parametrize('t1, t2, item_path, verbose_level',
@@ -261,7 +268,7 @@ class TestDeepDiffPretty:
                              ])
     def test_pretty_print_diff_iterable_item_removed(self, t1, t2, item_path, verbose_level):
         ddiff = DeepDiff(t1, t2, view='tree', verbose_level=verbose_level)
-        result = pretty_print_diff(ddiff.tree['iterable_item_removed'].items[0])
+        result = pretty_print_diff(ddiff.tree['iterable_item_removed'][0])
         assert result == 'Item {} removed from iterable.'.format(item_path)
 
     @pytest.mark.parametrize("verbose_level", range(3))
@@ -271,7 +278,7 @@ class TestDeepDiffPretty:
         t2.two = 2
 
         ddiff = DeepDiff(t1, t2, view='tree', verbose_level=verbose_level)
-        result = pretty_print_diff(ddiff.tree['attribute_added'].items[0])
+        result = pretty_print_diff(ddiff.tree['attribute_added'][0])
         assert result == 'Attribute root.two (2) added.' if verbose_level == 2 else 'Attribute root.two added.'
 
     @pytest.mark.parametrize("verbose_level", range(3))
@@ -281,7 +288,7 @@ class TestDeepDiffPretty:
         t2 = self.testing_class()
 
         ddiff = DeepDiff(t1, t2, view='tree', verbose_level=verbose_level)
-        result = pretty_print_diff(ddiff.tree['attribute_removed'].items[0])
+        result = pretty_print_diff(ddiff.tree['attribute_removed'][0])
 
         assert result == 'Attribute root.two (2) removed.' if verbose_level == 2 else 'Attribute root.two removed.'
 
@@ -291,7 +298,7 @@ class TestDeepDiffPretty:
                              ])
     def test_pretty_print_diff_set_item_added(self, t1, t2, item_path):
         ddiff = DeepDiff(t1, t2, view='tree')
-        result = pretty_print_diff(ddiff.tree['set_item_added'].items[0])
+        result = pretty_print_diff(ddiff.tree['set_item_added'][0])
         assert result == 'Item {} added to set.'.format(item_path)
 
     @pytest.mark.parametrize('t1, t2, item_path',
@@ -300,7 +307,7 @@ class TestDeepDiffPretty:
                              ])
     def test_pretty_print_diff_set_item_removed(self, t1, t2, item_path):
         ddiff = DeepDiff(t1, t2, view='tree')
-        result = pretty_print_diff(ddiff.tree['set_item_removed'].items[0])
+        result = pretty_print_diff(ddiff.tree['set_item_removed'][0])
         assert result == 'Item {} removed from set.'.format(item_path)
 
     @pytest.mark.parametrize('t1, t2, item_path',
@@ -309,7 +316,7 @@ class TestDeepDiffPretty:
                              ])
     def test_pretty_print_diff_repetition_change(self, t1, t2, item_path):
         ddiff = DeepDiff(t1, t2, view='tree', ignore_order=True, report_repetition=True)
-        result = pretty_print_diff(ddiff.tree['repetition_change'].items[0])
+        result = pretty_print_diff(ddiff.tree['repetition_change'][0])
         assert result == 'Repetition change for item {}.'.format(item_path)
 
     @pytest.mark.parametrize("expected, verbose_level",
@@ -331,6 +338,49 @@ class TestDeepDiffPretty:
         result = ddiff.pretty()
         assert result == expected
 
+    @pytest.mark.parametrize("expected, verbose_level",
+                             (
+                                 ('\t\tItem root[5] added to dictionary.'
+                                  '\n\t\tItem root[3] removed from dictionary.'
+                                  '\n\t\tType of root[2] changed from int to str and value changed from 2 to "b".'
+                                  '\n\t\tValue of root[4] changed from 4 to 5.', 0),
+                                 ('\t\tItem root[5] (5) added to dictionary.'
+                                  '\n\t\tItem root[3] (3) removed from dictionary.'
+                                  '\n\t\tType of root[2] changed from int to str and value changed from 2 to "b".'
+                                  '\n\t\tValue of root[4] changed from 4 to 5.', 2),
+                             ), ids=("verbose=0", "verbose=2")
+                             )
+    def test_pretty_form_method_prefixed_simple(self, expected, verbose_level):
+        t1 = {2: 2, 3: 3, 4: 4}
+        t2 = {2: 'b', 4: 5, 5: 5}
+        ddiff = DeepDiff(t1, t2, verbose_level=verbose_level)
+        result = ddiff.pretty(prefix="\t\t")
+        assert result == expected
+
+    @pytest.mark.parametrize("expected, verbose_level",
+                             (
+                                 ('Diff #1: Item root[5] added to dictionary.'
+                                  '\nDiff #2: Item root[3] removed from dictionary.'
+                                  '\nDiff #3: Type of root[2] changed from int to str and value changed from 2 to "b".'
+                                  '\nDiff #4: Value of root[4] changed from 4 to 5.', 0),
+                                 ('Diff #1: Item root[5] (5) added to dictionary.'
+                                  '\nDiff #2: Item root[3] (3) removed from dictionary.'
+                                  '\nDiff #3: Type of root[2] changed from int to str and value changed from 2 to "b".'
+                                  '\nDiff #4: Value of root[4] changed from 4 to 5.', 2),
+                             ), ids=("verbose=0", "verbose=2")
+                             )
+    def test_pretty_form_method_prefixed_callback(self, expected, verbose_level):
+        def prefix_callback(**kwargs):
+            """Helper function using a hidden variable on the diff that tracks which count prints next"""
+            kwargs['diff']._diff_count = 1 + getattr(kwargs['diff'], '_diff_count', 0)
+            return f"Diff #{kwargs['diff']._diff_count}: "
+
+        t1 = {2: 2, 3: 3, 4: 4}
+        t2 = {2: 'b', 4: 5, 5: 5}
+        ddiff = DeepDiff(t1, t2, verbose_level=verbose_level)
+        result = ddiff.pretty(prefix=prefix_callback)
+        assert result == expected
+
     @pytest.mark.parametrize('test_num, value, func_to_convert_back', [
         (1, {'10': None}, None),
         (2, {"type_changes": {"root": {"old_type": None, "new_type": list, "new_value": ["你好", 2, 3, 5]}}}, None),
@@ -360,3 +410,13 @@ class TestDeepDiffPretty:
         serialized = json_dumps(op_code)
         expected = '{"tag":"replace","t1_from_index":0,"t1_to_index":1,"t2_from_index":10,"t2_to_index":20,"old_values":null,"new_values":null}'
         assert serialized == expected
+
+    def test_reversed_list(self):
+        items = reversed([1, 2, 3])
+
+        serialized = json_dumps(items)
+        serialized2 = json_dumps(items)
+
+        assert '[3,2,1]' == serialized
+        assert '[3,2,1]' == serialized2, "We should have copied the original list. If this returns empty, it means we exhausted the original list."
+
